package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.AlgorithmBo;
import org.eco.als.domain.bo.TaskBo;
import org.eco.als.service.IAlgorithmService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 算法相关控制
 *
 * @Author: GaoKun Wang
 * @Date: 2024/7/30
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/algorithm")
public class AlgorithmController {
    @Resource
    private IAlgorithmService algorithmService;

    /**
     * 执行预处理
     *
     * @param algorithmBo 入参
     * @return org.eco.common.core.core.domain.CommonResult<java.lang.String> 结果
     **/
    @SaCheckPermission("als:algorithm:execute")
    @Log(title = "执行预处理算法", businessType = BusinessType.EXECUTE)
    @PostMapping("/execute/pro")
    public CommonResult<String> executePro(@Validated @RequestBody AlgorithmBo algorithmBo) {
        return StrUtil.isBlank(algorithmService.executePro(algorithmBo)) ? CommonResult.fail() : CommonResult.success();
    }

    /**
     * 执行虚警抑制
     *
     * @param algorithmBo 入参
     * @return org.eco.common.core.core.domain.CommonResult<java.lang.String> 结果
     **/
    @SaCheckPermission("als:algorithm:execute")
    @Log(title = "执行虚警抑制算法", businessType = BusinessType.EXECUTE)
    @PostMapping("/execute/falseAlarm")
    public CommonResult<String> executeFalseAlarm(@Validated @RequestBody AlgorithmBo algorithmBo) {
        return StrUtil.isBlank(algorithmService.executeFalseAlarm(algorithmBo)) ? CommonResult.fail() : CommonResult.success();
    }

    /**
     * 执行古战诊断
     *
     * @param algorithmBo 入参
     * @return org.eco.common.core.core.domain.CommonResult<java.lang.String> 结果
     **/
    @SaCheckPermission("als:algorithm:execute")
    @Log(title = "执行故障诊断算法", businessType = BusinessType.EXECUTE)
    @PostMapping("/execute/fault")
    public CommonResult<String> executeFault(@Validated @RequestBody AlgorithmBo algorithmBo) {
        return StrUtil.isBlank(algorithmService.executeFault(algorithmBo)) ? CommonResult.fail() : CommonResult.success();
    }

    /**
     * 执行任务管理
     *
     * @param taskBo 入参
     * @return org.eco.common.core.core.domain.CommonResult<java.lang.String> 结果
     **/
    @SaCheckPermission("als:algorithm:execute")
    @Log(title = "执行任务", businessType = BusinessType.EXECUTE)
    @PostMapping("/execute/task")
    public CommonResult<String> batchExecute(@Validated @RequestBody TaskBo taskBo) {
        return StrUtil.isBlank(algorithmService.batchExecute(taskBo)) ? CommonResult.fail() : CommonResult.success();
    }
}
