package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultConsumableImportVo;
import org.eco.als.domain.vo.FaultConsumableVo;
import org.eco.als.domain.bo.FaultConsumableBo;
import org.eco.als.service.IFaultConsumableService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障所需耗材Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultConsumable")
public class FaultConsumableController extends BaseController {
    @Resource
    private IFaultConsumableService faultConsumableService;

    /**
     * 查询故障所需耗材列表
     */
    @SaCheckPermission("als:faultConsumable:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultConsumableVo>> list(FaultConsumableBo faultConsumableBo) {
        return CommonResult.success(faultConsumableService.selectPage(faultConsumableBo));
    }

    /**
     * 导出故障所需耗材列表
     */
    @SaCheckPermission("als:faultConsumable:export")
    @Log(title = "故障所需耗材", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultConsumableBo faultConsumableBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultConsumableVo> list = faultConsumableService.selectList(faultConsumableBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultConsumableService.asyncExport(list, "故障所需耗材", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultConsumable:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障所需耗材", FaultConsumableImportVo.class, response);
    }

    /**
     * 导入故障所需耗材列表
     */
    @Log(title = "故障所需耗材", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultConsumable:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultConsumableService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障所需耗材详细信息
     */
    @SaCheckPermission("als:faultConsumable:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultConsumableVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultConsumableService.selectById(id));
    }

    /**
     * 新增故障所需耗材
     */
    @SaCheckPermission("als:faultConsumable:add")
    @Log(title = "故障所需耗材", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultConsumableBo faultConsumableBo) {
        boolean inserted = faultConsumableService.insert(faultConsumableBo);
        if (!inserted) {
            return CommonResult.fail("新增故障所需耗材记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障所需耗材
     */
    @SaCheckPermission("als:faultConsumable:edit")
    @Log(title = "故障所需耗材", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultConsumableBo faultConsumableBo) {
        boolean updated = faultConsumableService.update(faultConsumableBo);
        if (!updated) {
            return CommonResult.fail("修改故障所需耗材记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障所需耗材
     */
    @SaCheckPermission("als:faultConsumable:remove")
    @Log(title = "故障所需耗材", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultConsumableService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障所需耗材记录失败!");
        }
        return CommonResult.success();
    }
}
