package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultEquipmentImportVo;
import org.eco.als.domain.vo.FaultEquipmentVo;
import org.eco.als.domain.bo.FaultEquipmentBo;
import org.eco.als.service.IFaultEquipmentService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障所需设备Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultEquipment")
public class FaultEquipmentController extends BaseController {
    @Resource
    private IFaultEquipmentService faultEquipmentService;

    /**
     * 查询故障所需设备列表
     */
    @SaCheckPermission("als:faultEquipment:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultEquipmentVo>> list(FaultEquipmentBo faultEquipmentBo) {
        return CommonResult.success(faultEquipmentService.selectPage(faultEquipmentBo));
    }

    /**
     * 导出故障所需设备列表
     */
    @SaCheckPermission("als:faultEquipment:export")
    @Log(title = "故障所需设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultEquipmentBo faultEquipmentBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultEquipmentVo> list = faultEquipmentService.selectList(faultEquipmentBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultEquipmentService.asyncExport(list, "故障所需设备", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultEquipment:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障所需设备", FaultEquipmentImportVo.class, response);
    }

    /**
     * 导入故障所需设备列表
     */
    @Log(title = "故障所需设备", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultEquipment:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultEquipmentService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障所需设备详细信息
     */
    @SaCheckPermission("als:faultEquipment:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultEquipmentVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultEquipmentService.selectById(id));
    }

    /**
     * 新增故障所需设备
     */
    @SaCheckPermission("als:faultEquipment:add")
    @Log(title = "故障所需设备", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultEquipmentBo faultEquipmentBo) {
        boolean inserted = faultEquipmentService.insert(faultEquipmentBo);
        if (!inserted) {
            return CommonResult.fail("新增故障所需设备记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障所需设备
     */
    @SaCheckPermission("als:faultEquipment:edit")
    @Log(title = "故障所需设备", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultEquipmentBo faultEquipmentBo) {
        boolean updated = faultEquipmentService.update(faultEquipmentBo);
        if (!updated) {
            return CommonResult.fail("修改故障所需设备记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障所需设备
     */
    @SaCheckPermission("als:faultEquipment:remove")
    @Log(title = "故障所需设备", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultEquipmentService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障所需设备记录失败!");
        }
        return CommonResult.success();
    }
}
