package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultNotesImportVo;
import org.eco.als.domain.vo.FaultNotesVo;
import org.eco.als.domain.bo.FaultNotesBo;
import org.eco.als.service.IFaultNotesService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障注意事项Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultNotes")
public class FaultNotesController extends BaseController {
    @Resource
    private IFaultNotesService faultNotesService;

    /**
     * 查询故障注意事项列表
     */
    @SaCheckPermission("als:faultNotes:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultNotesVo>> list(FaultNotesBo faultNotesBo) {
        return CommonResult.success(faultNotesService.selectPage(faultNotesBo));
    }

    /**
     * 导出故障注意事项列表
     */
    @SaCheckPermission("als:faultNotes:export")
    @Log(title = "故障注意事项", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultNotesBo faultNotesBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultNotesVo> list = faultNotesService.selectList(faultNotesBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultNotesService.asyncExport(list, "故障注意事项", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultNotes:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障注意事项", FaultNotesImportVo.class, response);
    }

    /**
     * 导入故障注意事项列表
     */
    @Log(title = "故障注意事项", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultNotes:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultNotesService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障注意事项详细信息
     */
    @SaCheckPermission("als:faultNotes:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultNotesVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultNotesService.selectById(id));
    }

    /**
     * 新增故障注意事项
     */
    @SaCheckPermission("als:faultNotes:add")
    @Log(title = "故障注意事项", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultNotesBo faultNotesBo) {
        boolean inserted = faultNotesService.insert(faultNotesBo);
        if (!inserted) {
            return CommonResult.fail("新增故障注意事项记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障注意事项
     */
    @SaCheckPermission("als:faultNotes:edit")
    @Log(title = "故障注意事项", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultNotesBo faultNotesBo) {
        boolean updated = faultNotesService.update(faultNotesBo);
        if (!updated) {
            return CommonResult.fail("修改故障注意事项记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障注意事项
     */
    @SaCheckPermission("als:faultNotes:remove")
    @Log(title = "故障注意事项", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultNotesService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障注意事项记录失败!");
        }
        return CommonResult.success();
    }
}
