package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultPersonnelImportVo;
import org.eco.als.domain.vo.FaultPersonnelVo;
import org.eco.als.domain.bo.FaultPersonnelBo;
import org.eco.als.service.IFaultPersonnelService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障操作人员Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultPersonnel")
public class FaultPersonnelController extends BaseController {
    @Resource
    private IFaultPersonnelService faultPersonnelService;

    /**
     * 查询故障操作人员列表
     */
    @SaCheckPermission("als:faultPersonnel:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultPersonnelVo>> list(FaultPersonnelBo faultPersonnelBo) {
        return CommonResult.success(faultPersonnelService.selectPage(faultPersonnelBo));
    }

    /**
     * 导出故障操作人员列表
     */
    @SaCheckPermission("als:faultPersonnel:export")
    @Log(title = "故障操作人员", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultPersonnelBo faultPersonnelBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultPersonnelVo> list = faultPersonnelService.selectList(faultPersonnelBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultPersonnelService.asyncExport(list, "故障操作人员", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultPersonnel:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障操作人员", FaultPersonnelImportVo.class, response);
    }

    /**
     * 导入故障操作人员列表
     */
    @Log(title = "故障操作人员", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultPersonnel:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultPersonnelService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障操作人员详细信息
     */
    @SaCheckPermission("als:faultPersonnel:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultPersonnelVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultPersonnelService.selectById(id));
    }

    /**
     * 新增故障操作人员
     */
    @SaCheckPermission("als:faultPersonnel:add")
    @Log(title = "故障操作人员", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultPersonnelBo faultPersonnelBo) {
        boolean inserted = faultPersonnelService.insert(faultPersonnelBo);
        if (!inserted) {
            return CommonResult.fail("新增故障操作人员记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障操作人员
     */
    @SaCheckPermission("als:faultPersonnel:edit")
    @Log(title = "故障操作人员", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultPersonnelBo faultPersonnelBo) {
        boolean updated = faultPersonnelService.update(faultPersonnelBo);
        if (!updated) {
            return CommonResult.fail("修改故障操作人员记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障操作人员
     */
    @SaCheckPermission("als:faultPersonnel:remove")
    @Log(title = "故障操作人员", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultPersonnelService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障操作人员记录失败!");
        }
        return CommonResult.success();
    }
}
