package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultSetpsImportVo;
import org.eco.als.domain.vo.FaultSetpsVo;
import org.eco.als.domain.bo.FaultSetpsBo;
import org.eco.als.service.IFaultSetpsService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障操作步骤Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultSetps")
public class FaultSetpsController extends BaseController {
    @Resource
    private IFaultSetpsService faultSetpsService;

    /**
     * 查询故障操作步骤列表
     */
    @SaCheckPermission("als:faultSetps:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultSetpsVo>> list(FaultSetpsBo faultSetpsBo) {
        return CommonResult.success(faultSetpsService.selectPage(faultSetpsBo));
    }

    /**
     * 导出故障操作步骤列表
     */
    @SaCheckPermission("als:faultSetps:export")
    @Log(title = "故障操作步骤", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultSetpsBo faultSetpsBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultSetpsVo> list = faultSetpsService.selectList(faultSetpsBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultSetpsService.asyncExport(list, "故障操作步骤", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultSetps:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障操作步骤", FaultSetpsImportVo.class, response);
    }

    /**
     * 导入故障操作步骤列表
     */
    @Log(title = "故障操作步骤", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultSetps:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultSetpsService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障操作步骤详细信息
     */
    @SaCheckPermission("als:faultSetps:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultSetpsVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultSetpsService.selectById(id));
    }

    /**
     * 新增故障操作步骤
     */
    @SaCheckPermission("als:faultSetps:add")
    @Log(title = "故障操作步骤", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultSetpsBo faultSetpsBo) {
        boolean inserted = faultSetpsService.insert(faultSetpsBo);
        if (!inserted) {
            return CommonResult.fail("新增故障操作步骤记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障操作步骤
     */
    @SaCheckPermission("als:faultSetps:edit")
    @Log(title = "故障操作步骤", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultSetpsBo faultSetpsBo) {
        boolean updated = faultSetpsService.update(faultSetpsBo);
        if (!updated) {
            return CommonResult.fail("修改故障操作步骤记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障操作步骤
     */
    @SaCheckPermission("als:faultSetps:remove")
    @Log(title = "故障操作步骤", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultSetpsService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障操作步骤记录失败!");
        }
        return CommonResult.success();
    }
}
