package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.FaultSparePartsImportVo;
import org.eco.als.domain.vo.FaultSparePartsVo;
import org.eco.als.domain.bo.FaultSparePartsBo;
import org.eco.als.service.IFaultSparePartsService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 故障所需备件Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultSpareParts")
public class FaultSparePartsController extends BaseController {
    @Resource
    private IFaultSparePartsService faultSparePartsService;

    /**
     * 查询故障所需备件列表
     */
    @SaCheckPermission("als:faultSpareParts:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultSparePartsVo>> list(FaultSparePartsBo faultSparePartsBo) {
        return CommonResult.success(faultSparePartsService.selectPage(faultSparePartsBo));
    }

    /**
     * 导出故障所需备件列表
     */
    @SaCheckPermission("als:faultSpareParts:export")
    @Log(title = "故障所需备件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultSparePartsBo faultSparePartsBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultSparePartsVo> list = faultSparePartsService.selectList(faultSparePartsBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultSparePartsService.asyncExport(list, "故障所需备件", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:faultSpareParts:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "故障所需备件", FaultSparePartsImportVo.class, response);
    }

    /**
     * 导入故障所需备件列表
     */
    @Log(title = "故障所需备件", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultSpareParts:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        faultSparePartsService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障所需备件详细信息
     */
    @SaCheckPermission("als:faultSpareParts:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultSparePartsVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultSparePartsService.selectById(id));
    }

    /**
     * 新增故障所需备件
     */
    @SaCheckPermission("als:faultSpareParts:add")
    @Log(title = "故障所需备件", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultSparePartsBo faultSparePartsBo) {
        boolean inserted = faultSparePartsService.insert(faultSparePartsBo);
        if (!inserted) {
            return CommonResult.fail("新增故障所需备件记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障所需备件
     */
    @SaCheckPermission("als:faultSpareParts:edit")
    @Log(title = "故障所需备件", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultSparePartsBo faultSparePartsBo) {
        boolean updated = faultSparePartsService.update(faultSparePartsBo);
        if (!updated) {
            return CommonResult.fail("修改故障所需备件记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障所需备件
     */
    @SaCheckPermission("als:faultSpareParts:remove")
    @Log(title = "故障所需备件", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultSparePartsService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障所需备件记录失败!");
        }
        return CommonResult.success();
    }
}
