package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.MaintenanceTaskBo;
import org.eco.als.domain.vo.MaintenanceTaskVo;
import org.eco.als.service.IMaintenanceTaskService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 维修任务Controller
 *
 * @author wgk
 * @date 2024-07-24
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/maintenanceTask")
public class MaintenanceTaskController extends BaseController {
    @Resource
    private IMaintenanceTaskService maintenanceTaskService;

    /**
     * 查询维修任务列表
     */
    @SaCheckPermission("als:maintenanceTask:list")
    @GetMapping("/list")
    public CommonResult<PageResult<MaintenanceTaskVo>> list(MaintenanceTaskBo maintenanceTaskBo) {
        return CommonResult.success(maintenanceTaskService.selectPage(maintenanceTaskBo));
    }

    /**
     * 导出维修任务列表
     */
    @SaCheckPermission("als:maintenanceTask:export")
    @Log(title = "维修任务", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(MaintenanceTaskBo maintenanceTaskBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<MaintenanceTaskVo> list = maintenanceTaskService.selectList(maintenanceTaskBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        maintenanceTaskService.asyncExport(list, "维修任务", loginUser);
        return CommonResult.success();
    }

    /**
     * 获取维修任务详细信息
     */
    @SaCheckPermission("als:maintenanceTask:query")
    @GetMapping(value = "/{id}")
    public CommonResult<MaintenanceTaskVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(maintenanceTaskService.selectById(id));
    }

    /**
     * 新增维修任务
     */
    @SaCheckPermission("als:maintenanceTask:add")
    @Log(title = "维修任务", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody MaintenanceTaskBo maintenanceTaskBo) {
        boolean inserted = maintenanceTaskService.insert(maintenanceTaskBo);
        if (!inserted) {
            return CommonResult.fail("新增维修任务记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改维修任务
     */
    @SaCheckPermission("als:maintenanceTask:edit")
    @Log(title = "维修任务", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody MaintenanceTaskBo maintenanceTaskBo) {
        boolean updated = maintenanceTaskService.update(maintenanceTaskBo);
        if (!updated) {
            return CommonResult.fail("修改维修任务记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除维修任务
     */
    @SaCheckPermission("als:maintenanceTask:remove")
    @Log(title = "维修任务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = maintenanceTaskService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除维修任务记录失败!");
        }
        return CommonResult.success();
    }
}
