package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.SmTaskSubBo;
import org.eco.als.domain.vo.SmTaskSubImportVo;
import org.eco.als.domain.vo.SmTaskSubVo;
import org.eco.als.service.ISmTaskSubService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 定检任务子Controller
 *
 * @author wgk
 * @date 2024-07-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/smTaskSub")
public class SmTaskSubController extends BaseController {
    @Resource
    private ISmTaskSubService smTaskSubService;

    /**
     * 查询定检任务子列表
     */
    @SaCheckPermission("als:smTaskSub:list")
    @GetMapping("/list")
    public CommonResult<PageResult<SmTaskSubVo>> list(SmTaskSubBo smTaskSubBo) {
        return CommonResult.success(smTaskSubService.selectPage(smTaskSubBo));
    }

    /**
     * 导出定检任务子列表
     */
    @SaCheckPermission("als:smTaskSub:export")
    @Log(title = "定检任务子", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(SmTaskSubBo smTaskSubBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<SmTaskSubVo> list = smTaskSubService.selectList(smTaskSubBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        smTaskSubService.asyncExport(list, "定检任务子", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:smTaskSub:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "定检任务子", SmTaskSubImportVo.class, response);
    }

    /**
     * 导入定检任务子列表
     */
    @Log(title = "定检任务子", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:smTaskSub:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        smTaskSubService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取定检任务子详细信息
     */
    @SaCheckPermission("als:smTaskSub:query")
    @GetMapping(value = "/{id}")
    public CommonResult<SmTaskSubVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(smTaskSubService.selectById(id));
    }

    /**
     * 新增定检任务子
     */
    @SaCheckPermission("als:smTaskSub:add")
    @Log(title = "定检任务子", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody SmTaskSubBo smTaskSubBo) {
        boolean inserted = smTaskSubService.insert(smTaskSubBo);
        if (!inserted) {
            return CommonResult.fail("新增定检任务子记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改定检任务子
     */
    @SaCheckPermission("als:smTaskSub:edit")
    @Log(title = "定检任务子", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody SmTaskSubBo smTaskSubBo) {
        boolean updated = smTaskSubService.update(smTaskSubBo);
        if (!updated) {
            return CommonResult.fail("修改定检任务子记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除定检任务子
     */
    @SaCheckPermission("als:smTaskSub:remove")
    @Log(title = "定检任务子", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = smTaskSubService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除定检任务子记录失败!");
        }
        return CommonResult.success();
    }
}
