package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultConsumableMapper;
import org.eco.als.domain.FaultConsumable;
import org.eco.als.domain.bo.FaultConsumableBo;
import org.eco.als.domain.vo.FaultConsumableVo;
import org.eco.als.domain.vo.FaultConsumableImportVo;
import org.eco.als.listener.FaultConsumableImportListener;
import org.eco.als.service.IFaultConsumableService;

import static org.eco.als.domain.table.FaultConsumableTableDef.FAULT_CONSUMABLE;

/**
 * 故障所需耗材Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultConsumableServiceImpl extends BaseServiceImpl<FaultConsumableMapper, FaultConsumable> implements IFaultConsumableService {
    @Resource
    private FaultConsumableMapper faultConsumableMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_CONSUMABLE);
    }

    private QueryWrapper buildQueryWrapper(FaultConsumableBo faultConsumableBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_CONSUMABLE.CONSUMABLE_NAME.like
            (faultConsumableBo.getConsumableName()));
        queryWrapper.and(FAULT_CONSUMABLE.QUANTITY.eq
            (faultConsumableBo.getQuantity()));
        queryWrapper.and(FAULT_CONSUMABLE.PURPOSE.eq
            (faultConsumableBo.getPurpose()));
        queryWrapper.and(FAULT_CONSUMABLE.REMARKS.eq
            (faultConsumableBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障所需耗材
     *
     * @param id 故障所需耗材主键
     * @return 故障所需耗材
     */
    @Override
    public FaultConsumableVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_CONSUMABLE.ID.eq(id)), FaultConsumableVo.class);

    }


    /**
     * 查询故障所需耗材列表
     *
     * @param faultConsumableBo 故障所需耗材Bo
     * @return 故障所需耗材集合
     */
    @Override
    public List<FaultConsumableVo> selectList(FaultConsumableBo faultConsumableBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultConsumableBo);
        return this.listAs(queryWrapper, FaultConsumableVo.class);
    }

    /**
     * 分页查询故障所需耗材列表
     *
     * @param faultConsumableBo 故障所需耗材Bo
     * @return 分页故障所需耗材集合
     */
    @Override
    public PageResult<FaultConsumableVo> selectPage(FaultConsumableBo faultConsumableBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultConsumableBo);
        Page<FaultConsumableVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultConsumableVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障所需耗材
     *
     * @param faultConsumableBo 故障所需耗材Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultConsumableBo faultConsumableBo) {
        FaultConsumable faultConsumable = MapstructUtils.convert(faultConsumableBo, FaultConsumable.class);

        return this.save(faultConsumable);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障所需耗材，前台提供主键值，一般用于导入的场合
     *
     * @param faultConsumableBo 故障所需耗材Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultConsumableBo faultConsumableBo) {
        FaultConsumable faultConsumable = MapstructUtils.convert(faultConsumableBo, FaultConsumable.class);


        return faultConsumableMapper.insertWithPk(faultConsumable) > 0;//前台传来主键值
    }

    /**
     * 修改故障所需耗材
     *
     * @param faultConsumableBo 故障所需耗材Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultConsumableBo faultConsumableBo) {
        FaultConsumable faultConsumable = MapstructUtils.convert(faultConsumableBo, FaultConsumable.class);
        if (ObjectUtil.isNotNull(faultConsumable) && ObjectUtil.isNotNull(faultConsumable.getId())) {
            boolean updated = this.updateById(faultConsumable);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultConsumableImportVo.class, new FaultConsumableImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultConsumableVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultConsumableVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障所需耗材
     *
     * @param ids 需要删除的故障所需耗材主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
