package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultEquipmentMapper;
import org.eco.als.domain.FaultEquipment;
import org.eco.als.domain.bo.FaultEquipmentBo;
import org.eco.als.domain.vo.FaultEquipmentVo;
import org.eco.als.domain.vo.FaultEquipmentImportVo;
import org.eco.als.listener.FaultEquipmentImportListener;
import org.eco.als.service.IFaultEquipmentService;

import static org.eco.als.domain.table.FaultEquipmentTableDef.FAULT_EQUIPMENT;

/**
 * 故障所需设备Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultEquipmentServiceImpl extends BaseServiceImpl<FaultEquipmentMapper, FaultEquipment> implements IFaultEquipmentService {
    @Resource
    private FaultEquipmentMapper faultEquipmentMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_EQUIPMENT);
    }

    private QueryWrapper buildQueryWrapper(FaultEquipmentBo faultEquipmentBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_EQUIPMENT.EQUIPMENT_CODE.eq
            (faultEquipmentBo.getEquipmentCode()));
        queryWrapper.and(FAULT_EQUIPMENT.EQUIPMENT_NAME.like
            (faultEquipmentBo.getEquipmentName()));
        queryWrapper.and(FAULT_EQUIPMENT.MODEL_NUMBER.eq
            (faultEquipmentBo.getModelNumber()));
        queryWrapper.and(FAULT_EQUIPMENT.QUANTITY.eq
            (faultEquipmentBo.getQuantity()));
        queryWrapper.and(FAULT_EQUIPMENT.PURPOSE.eq
            (faultEquipmentBo.getPurpose()));
        queryWrapper.and(FAULT_EQUIPMENT.REMARKS.eq
            (faultEquipmentBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障所需设备
     *
     * @param id 故障所需设备主键
     * @return 故障所需设备
     */
    @Override
    public FaultEquipmentVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_EQUIPMENT.ID.eq(id)), FaultEquipmentVo.class);

    }


    /**
     * 查询故障所需设备列表
     *
     * @param faultEquipmentBo 故障所需设备Bo
     * @return 故障所需设备集合
     */
    @Override
    public List<FaultEquipmentVo> selectList(FaultEquipmentBo faultEquipmentBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultEquipmentBo);
        return this.listAs(queryWrapper, FaultEquipmentVo.class);
    }

    /**
     * 分页查询故障所需设备列表
     *
     * @param faultEquipmentBo 故障所需设备Bo
     * @return 分页故障所需设备集合
     */
    @Override
    public PageResult<FaultEquipmentVo> selectPage(FaultEquipmentBo faultEquipmentBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultEquipmentBo);
        Page<FaultEquipmentVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultEquipmentVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障所需设备
     *
     * @param faultEquipmentBo 故障所需设备Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultEquipmentBo faultEquipmentBo) {
        FaultEquipment faultEquipment = MapstructUtils.convert(faultEquipmentBo, FaultEquipment.class);

        return this.save(faultEquipment);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障所需设备，前台提供主键值，一般用于导入的场合
     *
     * @param faultEquipmentBo 故障所需设备Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultEquipmentBo faultEquipmentBo) {
        FaultEquipment faultEquipment = MapstructUtils.convert(faultEquipmentBo, FaultEquipment.class);


        return faultEquipmentMapper.insertWithPk(faultEquipment) > 0;//前台传来主键值
    }

    /**
     * 修改故障所需设备
     *
     * @param faultEquipmentBo 故障所需设备Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultEquipmentBo faultEquipmentBo) {
        FaultEquipment faultEquipment = MapstructUtils.convert(faultEquipmentBo, FaultEquipment.class);
        if (ObjectUtil.isNotNull(faultEquipment) && ObjectUtil.isNotNull(faultEquipment.getId())) {
            boolean updated = this.updateById(faultEquipment);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultEquipmentImportVo.class, new FaultEquipmentImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultEquipmentVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultEquipmentVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障所需设备
     *
     * @param ids 需要删除的故障所需设备主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
