package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultNotesMapper;
import org.eco.als.domain.FaultNotes;
import org.eco.als.domain.bo.FaultNotesBo;
import org.eco.als.domain.vo.FaultNotesVo;
import org.eco.als.domain.vo.FaultNotesImportVo;
import org.eco.als.listener.FaultNotesImportListener;
import org.eco.als.service.IFaultNotesService;

import static org.eco.als.domain.table.FaultNotesTableDef.FAULT_NOTES;

/**
 * 故障注意事项Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultNotesServiceImpl extends BaseServiceImpl<FaultNotesMapper, FaultNotes> implements IFaultNotesService {
    @Resource
    private FaultNotesMapper faultNotesMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_NOTES);
    }

    private QueryWrapper buildQueryWrapper(FaultNotesBo faultNotesBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_NOTES.NOTE_CONTENT.eq
            (faultNotesBo.getNoteContent()));
        queryWrapper.and(FAULT_NOTES.IMAGE.eq
            (faultNotesBo.getImage()));
        queryWrapper.and(FAULT_NOTES.REMARKS.eq
            (faultNotesBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障注意事项
     *
     * @param id 故障注意事项主键
     * @return 故障注意事项
     */
    @Override
    public FaultNotesVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_NOTES.ID.eq(id)), FaultNotesVo.class);

    }


    /**
     * 查询故障注意事项列表
     *
     * @param faultNotesBo 故障注意事项Bo
     * @return 故障注意事项集合
     */
    @Override
    public List<FaultNotesVo> selectList(FaultNotesBo faultNotesBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultNotesBo);
        return this.listAs(queryWrapper, FaultNotesVo.class);
    }

    /**
     * 分页查询故障注意事项列表
     *
     * @param faultNotesBo 故障注意事项Bo
     * @return 分页故障注意事项集合
     */
    @Override
    public PageResult<FaultNotesVo> selectPage(FaultNotesBo faultNotesBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultNotesBo);
        Page<FaultNotesVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultNotesVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障注意事项
     *
     * @param faultNotesBo 故障注意事项Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultNotesBo faultNotesBo) {
        FaultNotes faultNotes = MapstructUtils.convert(faultNotesBo, FaultNotes.class);

        return this.save(faultNotes);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障注意事项，前台提供主键值，一般用于导入的场合
     *
     * @param faultNotesBo 故障注意事项Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultNotesBo faultNotesBo) {
        FaultNotes faultNotes = MapstructUtils.convert(faultNotesBo, FaultNotes.class);


        return faultNotesMapper.insertWithPk(faultNotes) > 0;//前台传来主键值
    }

    /**
     * 修改故障注意事项
     *
     * @param faultNotesBo 故障注意事项Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultNotesBo faultNotesBo) {
        FaultNotes faultNotes = MapstructUtils.convert(faultNotesBo, FaultNotes.class);
        if (ObjectUtil.isNotNull(faultNotes) && ObjectUtil.isNotNull(faultNotes.getId())) {
            boolean updated = this.updateById(faultNotes);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultNotesImportVo.class, new FaultNotesImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultNotesVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultNotesVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障注意事项
     *
     * @param ids 需要删除的故障注意事项主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
