package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultPersonnelMapper;
import org.eco.als.domain.FaultPersonnel;
import org.eco.als.domain.bo.FaultPersonnelBo;
import org.eco.als.domain.vo.FaultPersonnelVo;
import org.eco.als.domain.vo.FaultPersonnelImportVo;
import org.eco.als.listener.FaultPersonnelImportListener;
import org.eco.als.service.IFaultPersonnelService;

import static org.eco.als.domain.table.FaultPersonnelTableDef.FAULT_PERSONNEL;

/**
 * 故障操作人员Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultPersonnelServiceImpl extends BaseServiceImpl<FaultPersonnelMapper, FaultPersonnel> implements IFaultPersonnelService {
    @Resource
    private FaultPersonnelMapper faultPersonnelMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_PERSONNEL);
    }

    private QueryWrapper buildQueryWrapper(FaultPersonnelBo faultPersonnelBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_PERSONNEL.PERSONNEL.eq
            (faultPersonnelBo.getPersonnel()));
        queryWrapper.and(FAULT_PERSONNEL.POSITION.eq
            (faultPersonnelBo.getPosition()));
        queryWrapper.and(FAULT_PERSONNEL.QUANTITY.eq
            (faultPersonnelBo.getQuantity()));
        queryWrapper.and(FAULT_PERSONNEL.DUTIES.eq
            (faultPersonnelBo.getDuties()));
        queryWrapper.and(FAULT_PERSONNEL.REMARKS.eq
            (faultPersonnelBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障操作人员
     *
     * @param id 故障操作人员主键
     * @return 故障操作人员
     */
    @Override
    public FaultPersonnelVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_PERSONNEL.ID.eq(id)), FaultPersonnelVo.class);

    }


    /**
     * 查询故障操作人员列表
     *
     * @param faultPersonnelBo 故障操作人员Bo
     * @return 故障操作人员集合
     */
    @Override
    public List<FaultPersonnelVo> selectList(FaultPersonnelBo faultPersonnelBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultPersonnelBo);
        return this.listAs(queryWrapper, FaultPersonnelVo.class);
    }

    /**
     * 分页查询故障操作人员列表
     *
     * @param faultPersonnelBo 故障操作人员Bo
     * @return 分页故障操作人员集合
     */
    @Override
    public PageResult<FaultPersonnelVo> selectPage(FaultPersonnelBo faultPersonnelBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultPersonnelBo);
        Page<FaultPersonnelVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultPersonnelVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障操作人员
     *
     * @param faultPersonnelBo 故障操作人员Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultPersonnelBo faultPersonnelBo) {
        FaultPersonnel faultPersonnel = MapstructUtils.convert(faultPersonnelBo, FaultPersonnel.class);

        return this.save(faultPersonnel);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障操作人员，前台提供主键值，一般用于导入的场合
     *
     * @param faultPersonnelBo 故障操作人员Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultPersonnelBo faultPersonnelBo) {
        FaultPersonnel faultPersonnel = MapstructUtils.convert(faultPersonnelBo, FaultPersonnel.class);


        return faultPersonnelMapper.insertWithPk(faultPersonnel) > 0;//前台传来主键值
    }

    /**
     * 修改故障操作人员
     *
     * @param faultPersonnelBo 故障操作人员Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultPersonnelBo faultPersonnelBo) {
        FaultPersonnel faultPersonnel = MapstructUtils.convert(faultPersonnelBo, FaultPersonnel.class);
        if (ObjectUtil.isNotNull(faultPersonnel) && ObjectUtil.isNotNull(faultPersonnel.getId())) {
            boolean updated = this.updateById(faultPersonnel);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultPersonnelImportVo.class, new FaultPersonnelImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultPersonnelVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultPersonnelVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障操作人员
     *
     * @param ids 需要删除的故障操作人员主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
