package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultSetpsMapper;
import org.eco.als.domain.FaultSetps;
import org.eco.als.domain.bo.FaultSetpsBo;
import org.eco.als.domain.vo.FaultSetpsVo;
import org.eco.als.domain.vo.FaultSetpsImportVo;
import org.eco.als.listener.FaultSetpsImportListener;
import org.eco.als.service.IFaultSetpsService;

import static org.eco.als.domain.table.FaultSetpsTableDef.FAULT_SETPS;

/**
 * 故障操作步骤Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultSetpsServiceImpl extends BaseServiceImpl<FaultSetpsMapper, FaultSetps> implements IFaultSetpsService {
    @Resource
    private FaultSetpsMapper faultSetpsMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_SETPS);
    }

    private QueryWrapper buildQueryWrapper(FaultSetpsBo faultSetpsBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_SETPS.SETPS.eq
            (faultSetpsBo.getSetps()));
        queryWrapper.and(FAULT_SETPS.METHOD.eq
            (faultSetpsBo.getMethod()));
        queryWrapper.and(FAULT_SETPS.IMAGE.eq
            (faultSetpsBo.getImage()));
        queryWrapper.and(FAULT_SETPS.REMARKS.eq
            (faultSetpsBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障操作步骤
     *
     * @param id 故障操作步骤主键
     * @return 故障操作步骤
     */
    @Override
    public FaultSetpsVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_SETPS.ID.eq(id)), FaultSetpsVo.class);

    }


    /**
     * 查询故障操作步骤列表
     *
     * @param faultSetpsBo 故障操作步骤Bo
     * @return 故障操作步骤集合
     */
    @Override
    public List<FaultSetpsVo> selectList(FaultSetpsBo faultSetpsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultSetpsBo);
        return this.listAs(queryWrapper, FaultSetpsVo.class);
    }

    /**
     * 分页查询故障操作步骤列表
     *
     * @param faultSetpsBo 故障操作步骤Bo
     * @return 分页故障操作步骤集合
     */
    @Override
    public PageResult<FaultSetpsVo> selectPage(FaultSetpsBo faultSetpsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultSetpsBo);
        Page<FaultSetpsVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultSetpsVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障操作步骤
     *
     * @param faultSetpsBo 故障操作步骤Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultSetpsBo faultSetpsBo) {
        FaultSetps faultSetps = MapstructUtils.convert(faultSetpsBo, FaultSetps.class);

        return this.save(faultSetps);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障操作步骤，前台提供主键值，一般用于导入的场合
     *
     * @param faultSetpsBo 故障操作步骤Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultSetpsBo faultSetpsBo) {
        FaultSetps faultSetps = MapstructUtils.convert(faultSetpsBo, FaultSetps.class);


        return faultSetpsMapper.insertWithPk(faultSetps) > 0;//前台传来主键值
    }

    /**
     * 修改故障操作步骤
     *
     * @param faultSetpsBo 故障操作步骤Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultSetpsBo faultSetpsBo) {
        FaultSetps faultSetps = MapstructUtils.convert(faultSetpsBo, FaultSetps.class);
        if (ObjectUtil.isNotNull(faultSetps) && ObjectUtil.isNotNull(faultSetps.getId())) {
            boolean updated = this.updateById(faultSetps);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultSetpsImportVo.class, new FaultSetpsImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultSetpsVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultSetpsVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障操作步骤
     *
     * @param ids 需要删除的故障操作步骤主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
