package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.FaultSparePartsMapper;
import org.eco.als.domain.FaultSpareParts;
import org.eco.als.domain.bo.FaultSparePartsBo;
import org.eco.als.domain.vo.FaultSparePartsVo;
import org.eco.als.domain.vo.FaultSparePartsImportVo;
import org.eco.als.listener.FaultSparePartsImportListener;
import org.eco.als.service.IFaultSparePartsService;

import static org.eco.als.domain.table.FaultSparePartsTableDef.FAULT_SPARE_PARTS;

/**
 * 故障所需备件Service业务层处理
 *
 * @author wgk
 * @date 2024-10-25
 */
@Service
@Slf4j
public class FaultSparePartsServiceImpl extends BaseServiceImpl<FaultSparePartsMapper, FaultSpareParts> implements IFaultSparePartsService {
    @Resource
    private FaultSparePartsMapper faultSparePartsMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_SPARE_PARTS);
    }

    private QueryWrapper buildQueryWrapper(FaultSparePartsBo faultSparePartsBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_SPARE_PARTS.SPARE_PARTS_CODE.eq
            (faultSparePartsBo.getSparePartsCode()));
        queryWrapper.and(FAULT_SPARE_PARTS.SPARE_PARTS_NAME.like
            (faultSparePartsBo.getSparePartsName()));
        queryWrapper.and(FAULT_SPARE_PARTS.MODEL_NUMBER.eq
            (faultSparePartsBo.getModelNumber()));
        queryWrapper.and(FAULT_SPARE_PARTS.QUANTITY.eq
            (faultSparePartsBo.getQuantity()));
        queryWrapper.and(FAULT_SPARE_PARTS.PURPOSE.eq
            (faultSparePartsBo.getPurpose()));
        queryWrapper.and(FAULT_SPARE_PARTS.REMARKS.eq
            (faultSparePartsBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障所需备件
     *
     * @param id 故障所需备件主键
     * @return 故障所需备件
     */
    @Override
    public FaultSparePartsVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_SPARE_PARTS.ID.eq(id)), FaultSparePartsVo.class);

    }


    /**
     * 查询故障所需备件列表
     *
     * @param faultSparePartsBo 故障所需备件Bo
     * @return 故障所需备件集合
     */
    @Override
    public List<FaultSparePartsVo> selectList(FaultSparePartsBo faultSparePartsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultSparePartsBo);
        return this.listAs(queryWrapper, FaultSparePartsVo.class);
    }

    /**
     * 分页查询故障所需备件列表
     *
     * @param faultSparePartsBo 故障所需备件Bo
     * @return 分页故障所需备件集合
     */
    @Override
    public PageResult<FaultSparePartsVo> selectPage(FaultSparePartsBo faultSparePartsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultSparePartsBo);
        Page<FaultSparePartsVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultSparePartsVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障所需备件
     *
     * @param faultSparePartsBo 故障所需备件Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultSparePartsBo faultSparePartsBo) {
        FaultSpareParts faultSpareParts = MapstructUtils.convert(faultSparePartsBo, FaultSpareParts.class);

        return this.save(faultSpareParts);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障所需备件，前台提供主键值，一般用于导入的场合
     *
     * @param faultSparePartsBo 故障所需备件Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultSparePartsBo faultSparePartsBo) {
        FaultSpareParts faultSpareParts = MapstructUtils.convert(faultSparePartsBo, FaultSpareParts.class);


        return faultSparePartsMapper.insertWithPk(faultSpareParts) > 0;//前台传来主键值
    }

    /**
     * 修改故障所需备件
     *
     * @param faultSparePartsBo 故障所需备件Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultSparePartsBo faultSparePartsBo) {
        FaultSpareParts faultSpareParts = MapstructUtils.convert(faultSparePartsBo, FaultSpareParts.class);
        if (ObjectUtil.isNotNull(faultSpareParts) && ObjectUtil.isNotNull(faultSpareParts.getId())) {
            boolean updated = this.updateById(faultSpareParts);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FaultSparePartsImportVo.class, new FaultSparePartsImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FaultSparePartsVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultSparePartsVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障所需备件
     *
     * @param ids 需要删除的故障所需备件主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
