package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.SmTaskRecord;
import org.eco.als.domain.bo.SmTaskRecordBo;
import org.eco.als.domain.vo.SmTaskRecordImportVo;
import org.eco.als.domain.vo.SmTaskRecordVo;
import org.eco.als.listener.SmTaskRecordImportListener;
import org.eco.als.mapper.SmTaskRecordMapper;
import org.eco.als.service.ISmTaskRecordService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.SmTaskRecordTableDef.SM_TASK_RECORD;

/**
 * 定检任务记录Service业务层处理
 *
 * @author wgk
 * @date 2024-07-25
 */
@Service
@Slf4j
public class SmTaskRecordServiceImpl extends BaseServiceImpl<SmTaskRecordMapper, SmTaskRecord> implements ISmTaskRecordService {
    @Resource
    private SmTaskRecordMapper smTaskRecordMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(SM_TASK_RECORD);
    }

    private QueryWrapper buildQueryWrapper(SmTaskRecordBo smTaskRecordBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(SM_TASK_RECORD.AIRCRAFT_ID.eq
            (smTaskRecordBo.getAircraftId()));
        queryWrapper.and(SM_TASK_RECORD.AIRCRAFT_MODEL.eq
            (smTaskRecordBo.getAircraftModel()));
        queryWrapper.and(SM_TASK_RECORD.NAME.like
            (smTaskRecordBo.getName()));
        queryWrapper.and(SM_TASK_RECORD.CODE.eq
            (smTaskRecordBo.getCode()));
        queryWrapper.and(SM_TASK_RECORD.END_TIME.eq
            (smTaskRecordBo.getEndTime()));
        queryWrapper.and(SM_TASK_RECORD.REMAIN_TIME.eq
            (smTaskRecordBo.getRemainTime()));
        queryWrapper.and(SM_TASK_RECORD.SPECIALTY.eq
            (smTaskRecordBo.getSpecialty()));
        queryWrapper.and(SM_TASK_RECORD.CONTENTS.eq
            (smTaskRecordBo.getContents()));

        return queryWrapper;
    }

    /**
     * 查询定检任务记录
     *
     * @param id 定检任务记录主键
     * @return 定检任务记录
     */
    @Override
    public SmTaskRecordVo selectById(Long id) {
        return this.getOneAs(query().where(SM_TASK_RECORD.ID.eq(id)), SmTaskRecordVo.class);

    }


    /**
     * 查询定检任务记录列表
     *
     * @param smTaskRecordBo 定检任务记录Bo
     * @return 定检任务记录集合
     */
    @Override
    public List<SmTaskRecordVo> selectList(SmTaskRecordBo smTaskRecordBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(smTaskRecordBo);
        return this.listAs(queryWrapper, SmTaskRecordVo.class);
    }

    /**
     * 分页查询定检任务记录列表
     *
     * @param smTaskRecordBo 定检任务记录Bo
     * @return 分页定检任务记录集合
     */
    @Override
    public PageResult<SmTaskRecordVo> selectPage(SmTaskRecordBo smTaskRecordBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(smTaskRecordBo);
        Page<SmTaskRecordVo> page = this.pageAs(PageQuery.build(), queryWrapper, SmTaskRecordVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增定检任务记录
     *
     * @param smTaskRecordBo 定检任务记录Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(SmTaskRecordBo smTaskRecordBo) {
        SmTaskRecord smTaskRecord = MapstructUtils.convert(smTaskRecordBo, SmTaskRecord.class);

        return this.save(smTaskRecord);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增定检任务记录，前台提供主键值，一般用于导入的场合
     *
     * @param smTaskRecordBo 定检任务记录Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(SmTaskRecordBo smTaskRecordBo) {
        SmTaskRecord smTaskRecord = MapstructUtils.convert(smTaskRecordBo, SmTaskRecord.class);


        return smTaskRecordMapper.insertWithPk(smTaskRecord) > 0;//前台传来主键值
    }

    /**
     * 修改定检任务记录
     *
     * @param smTaskRecordBo 定检任务记录Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(SmTaskRecordBo smTaskRecordBo) {
        SmTaskRecord smTaskRecord = MapstructUtils.convert(smTaskRecordBo, SmTaskRecord.class);
        if (ObjectUtil.isNotNull(smTaskRecord) && ObjectUtil.isNotNull(smTaskRecord.getId())) {
            return this.updateById(smTaskRecord);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, SmTaskRecordImportVo.class, new SmTaskRecordImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<SmTaskRecordVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, SmTaskRecordVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除定检任务记录
     *
     * @param ids 需要删除的定检任务记录主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
